import { DocumentData } from "firebase/firestore";
import { QueryReturn, andOrWhereClause, whereClause } from "./datatypes";
export interface StoreInterface {
    find(id: string): Promise<QueryReturn>;
    findAll(ids?: string[]): Promise<QueryReturn>;
    findWhereOrAnd({ wh, lim, order, offset }: {
        wh?: {
            type: 'or' | 'and' | 'andOr';
            parameter: andOrWhereClause[];
        };
        lim?: number;
        order?: string;
        offset?: string;
    }): Promise<QueryReturn>;
    findWhere({ wh, lim, order, offset }: {
        wh?: whereClause[];
        lim?: number;
        order?: string;
        offset?: string;
    }): Promise<QueryReturn>;
    save(data: object, id?: string): Promise<QueryReturn>;
    delete(id: string): Promise<QueryReturn>;
    update(data: object, id: string): Promise<QueryReturn>;
    stream(callBack: (data: DocumentData | DocumentData[] | undefined) => void, id?: string): void;
    streamWhere(wh: whereClause[], callBack: (data: DocumentData[]) => void, lim?: number, order?: string, offset?: string): void;
    countData(where: whereClause[]): Promise<QueryReturn>;
    saveBatch({ data }: {
        data: object[];
    }): Promise<QueryReturn>;
    updateBatch({ data }: {
        data: object[];
        callBack: () => void;
    }): Promise<QueryReturn>;
    deleteBatch({ ids }: {
        ids: string[];
    }): Promise<QueryReturn>;
    incrementDecrement({ dbReference, key, isIncrement, incrementalValue }: {
        dbReference: string;
        key: string;
        isIncrement?: boolean;
        incrementalValue?: number;
    }): Promise<QueryReturn>;
}
