import { StoreModel } from "./StoreModel";
import { deleteUser } from "firebase/auth";
export class UserModel extends StoreModel {
    _auth;
    constructor(auth, firestoreDB, table) {
        super(table ?? "Users", firestoreDB);
        this._auth = auth;
    }
    async registerWithEmailAndPassword({ email, password, userData }) {
        try {
            const user = await this._auth?.createUserWithEmailAndPassword(email, password);
            if (user && userData) {
                this.save(userData, user.user.uid);
            }
            return {
                data: user,
                status: 'success',
                message: 'User registered successfully'
            };
        }
        catch (error) {
            if (error.code === 'auth/email-already-in-use') {
                return {
                    status: 'error',
                    message: `${email} is already in use!`
                };
            }
            if (error.code === 'auth/invalid-email') {
                return {
                    status: 'error',
                    message: `${email} is an invalid email address`
                };
            }
            return {
                status: 'error',
                message: 'Unable to register user'
            };
        }
    }
    async login({ email, password, verifyEmail = false }) {
        try {
            const signedIn = await this._auth.signInWithEmailAndPassword(email, password);
            if (verifyEmail) {
                if (!signedIn.user.emailVerified) {
                    return {
                        status: 'error',
                        message: 'Email address needs verification'
                    };
                }
            }
            return {
                status: 'success',
                message: 'User logged in successfully',
                data: signedIn
            };
        }
        catch (error) {
            return {
                status: 'error',
                message: 'Invalid login credentials'
            };
        }
    }
    async logout() {
        try {
            await this._auth.signOut();
            return {
                status: 'success',
                message: "User signout!"
            };
        }
        catch (_) {
            return { status: 'error', message: 'Unable to complete process' };
        }
    }
    async deleteAccount(user) {
        try {
            await Promise.all([
                deleteUser(user),
                this.delete(user.uid)
            ]);
            return { status: 'success', message: 'Account deleted successfully' };
        }
        catch (error) {
            return { status: 'error', message: "unable to delete user account" };
        }
    }
}
//# sourceMappingURL=UserModel.js.map